@@ -36,6 +36,9 @@ public:
   virtual file_status_e read(generic_packetizer_c *ptzr, bool force = false);
   virtual void identify();
   virtual void create_packetizer(int64_t id);
+  virtual bool is_providing_timecodes() const {
+    return false;
+  }
 
   static int probe_file(mm_io_c *in, uint64_t size);
 };
