@@ -150,10 +150,10 @@ mpeg_es_reader_c::read_headers() {
     dheight = height;
 
     MPEGChunk *raw_seq_hdr = parser.GetRealSequenceHeader();
-    if (raw_seq_hdr) {
-      m_ti.m_private_data = (unsigned char *)safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
-      m_ti.m_private_size = raw_seq_hdr->GetSize();
-    }
+    if (raw_seq_hdr)
+      m_ti.m_private_data = memory_c::clone(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
+    else
+      m_ti.m_private_data.reset();
 
     mxverb(2, boost::format("mpeg_es_reader: version %1% width %2% height %3% FPS %4% AR %5%\n") % version % width % height % frame_rate % aspect_ratio);
 
