@@ -72,8 +72,6 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
   , m_default_track_warning_printed(false)
   , m_huid(0)
   , m_htrack_max_add_block_ids(-1)
-  , m_hcodec_private(nullptr)
-  , m_hcodec_private_length(0)
   , m_haudio_sampling_freq(-1.0)
   , m_haudio_output_sampling_freq(-1.0)
   , m_haudio_channels(-1)
@@ -120,6 +118,12 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
   else if (map_has_key(m_ti.m_default_track_flags, -1))
     m_ti.m_default_track = m_ti.m_default_track_flags[-1];
 
+  // Let's see if the user has given a fix avc fps flag for this track.
+  if (map_has_key(m_ti.m_fix_bitstream_frame_rate_flags, m_ti.m_id))
+    m_ti.m_fix_bitstream_frame_rate = m_ti.m_fix_bitstream_frame_rate_flags[m_ti.m_id];
+  else if (map_has_key(m_ti.m_fix_bitstream_frame_rate_flags, -1))
+    m_ti.m_fix_bitstream_frame_rate = m_ti.m_fix_bitstream_frame_rate_flags[-1];
+
   // Let's see if the user has given a forced track flag for this track.
   if (map_has_key(m_ti.m_forced_track_flags, m_ti.m_id))
     m_ti.m_forced_track = m_ti.m_forced_track_flags[m_ti.m_id];
@@ -246,7 +250,6 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
 }
 
 generic_packetizer_c::~generic_packetizer_c() {
-  safefree(m_hcodec_private);
   if (!m_packet_queue.empty())
     mxerror_tid(m_ti.m_fname, m_ti.m_id, boost::format(Y("Packet queue not empty (flushed: %1%). Frames have been lost during remux. %2%\n")) % m_has_been_flushed % BUGMSG);
 }
@@ -338,21 +341,15 @@ generic_packetizer_c::set_codec_id(const std::string &id) {
 }
 
 void
-generic_packetizer_c::set_codec_private(const unsigned char *cp,
-                                        int length) {
-  safefree(m_hcodec_private);
+generic_packetizer_c::set_codec_private(memory_cptr const &buffer) {
+  if (buffer && buffer->get_size()) {
+    m_hcodec_private = buffer->clone();
 
-  if (!cp) {
-    m_hcodec_private        = nullptr;
-    m_hcodec_private_length = 0;
-    return;
-  }
+    if (m_track_entry)
+      GetChild<KaxCodecPrivate>(*m_track_entry).CopyBuffer(static_cast<binary *>(m_hcodec_private->get_buffer()), m_hcodec_private->get_size());
 
-  m_hcodec_private        = (unsigned char *)safememdup(cp, length);
-  m_hcodec_private_length = length;
-
-  if (m_track_entry)
-    GetChild<KaxCodecPrivate>(*m_track_entry).CopyBuffer((binary *)m_hcodec_private, m_hcodec_private_length);
+  } else
+    m_hcodec_private.reset();
 }
 
 void
@@ -628,7 +625,7 @@ generic_packetizer_c::set_headers() {
     GetChild<KaxCodecID>(m_track_entry).SetValue(m_hcodec_id);
 
   if (m_hcodec_private)
-    GetChild<KaxCodecPrivate>(*m_track_entry).CopyBuffer((binary *)m_hcodec_private, m_hcodec_private_length);
+    GetChild<KaxCodecPrivate>(*m_track_entry).CopyBuffer(static_cast<binary *>(m_hcodec_private->get_buffer()), m_hcodec_private->get_size());
 
   if (!outputting_webm()) {
     if (-1 != m_htrack_min_cache)
@@ -1185,6 +1182,7 @@ generic_reader_c::generic_reader_c(const track_info_c &ti,
   add_all_requested_track_ids(timecode_sync_t,      m_timecode_syncs);
   add_all_requested_track_ids(cue_strategy_e,       m_cue_creations);
   add_all_requested_track_ids(bool,                 m_default_track_flags);
+  add_all_requested_track_ids(bool,                 m_fix_bitstream_frame_rate_flags);
   add_all_requested_track_ids(std::string,          m_languages);
   add_all_requested_track_ids(std::string,          m_sub_charsets);
   add_all_requested_track_ids(std::string,          m_all_tags);
@@ -1356,16 +1354,17 @@ generic_reader_c::flush_packetizers() {
 
 void
 generic_reader_c::id_result_container(const std::string &verbose_info) {
-  m_id_results_container.info = get_format_name().get_translated();
-  m_id_results_container.verbose_info.clear();
+  std::vector<std::string> verbose_info_list;
   if (!verbose_info.empty())
-    m_id_results_container.verbose_info.push_back(verbose_info);
+    verbose_info_list.push_back(verbose_info);
+  id_result_container(verbose_info_list);
 }
 
 void
 generic_reader_c::id_result_container(const std::vector<std::string> &verbose_info) {
   m_id_results_container.info         = get_format_name().get_translated();
   m_id_results_container.verbose_info = verbose_info;
+  m_id_results_container.verbose_info.push_back((boost::format("is_providing_timecodes:%1%") % (is_providing_timecodes() ? 1 : 0)).str());
 }
 
 void
@@ -1520,11 +1519,8 @@ generic_reader_c::get_underlying_input_as_multi_file_io()
 //--------------------------------------------------------------------
 
 track_info_c::track_info_c()
-  : m_initialized(true)
-  , m_id(0)
+  : m_id(0)
   , m_disable_multi_file(false)
-  , m_private_data(nullptr)
-  , m_private_size(0)
   , m_aspect_ratio(0.0)
   , m_display_width(0)
   , m_display_height(0)
@@ -1535,6 +1531,7 @@ track_info_c::track_info_c()
   , m_reset_timecodes(false)
   , m_cues(CUE_STRATEGY_UNSPECIFIED)
   , m_default_track(boost::logic::indeterminate)
+  , m_fix_bitstream_frame_rate(boost::logic::indeterminate)
   , m_forced_track(boost::logic::indeterminate)
   , m_enabled_track(boost::logic::indeterminate)
   , m_compression(COMPRESSION_UNSPECIFIED)
@@ -1553,20 +1550,8 @@ track_info_c::track_info_c()
 {
 }
 
-void
-track_info_c::free_contents() {
-  if (!m_initialized)
-    return;
-
-  safefree(m_private_data);
-
-  m_initialized = false;
-}
-
 track_info_c &
 track_info_c::operator =(const track_info_c &src) {
-  free_contents();
-
   m_id                         = src.m_id;
   m_fname                      = src.m_fname;
 
@@ -1577,8 +1562,7 @@ track_info_c::operator =(const track_info_c &src) {
   m_track_tags                 = src.m_track_tags;
   m_disable_multi_file         = src.m_disable_multi_file;
 
-  m_private_size               = src.m_private_size;
-  m_private_data               = (unsigned char *)safememdup(src.m_private_data, m_private_size);
+  m_private_data               = src.m_private_data ? src.m_private_data->clone() : src.m_private_data;
 
   m_all_fourccs                = src.m_all_fourccs;
 
@@ -1601,6 +1585,9 @@ track_info_c::operator =(const track_info_c &src) {
   m_default_track_flags        = src.m_default_track_flags;
   m_default_track              = src.m_default_track;
 
+  m_fix_bitstream_frame_rate_flags = src.m_fix_bitstream_frame_rate_flags;
+  m_fix_bitstream_frame_rate       = src.m_fix_bitstream_frame_rate;
+
   m_forced_track_flags         = src.m_forced_track_flags;
   m_forced_track               = src.m_forced_track;
 
@@ -1658,8 +1645,6 @@ track_info_c::operator =(const track_info_c &src) {
   m_default_durations          = src.m_default_durations;
   m_max_blockadd_ids           = src.m_max_blockadd_ids;
 
-  m_initialized                = true;
-
   return *this;
 }
 
